import 'dart:async';
import 'dart:convert';
import 'dart:core';
import 'dart:io';

import 'package:connectivity/connectivity.dart';
import 'package:crypto/crypto.dart';
import 'package:in_app_purchase/in_app_purchase.dart';
import 'package:in_app_purchase_android/billing_client_wrappers.dart';
import 'package:in_app_purchase_android/in_app_purchase_android.dart';
import 'package:in_app_purchase_storekit/in_app_purchase_storekit.dart';
import 'package:in_app_purchase_storekit/store_kit_wrappers.dart';
import 'package:mmoo_base/config/app_config.dart';
import 'package:mmoo_base/utils/common_utils.dart';
import 'package:mmoo_base_recharge/dao/recharge_dao.dart';
import 'package:mmoo_base_recharge/entity/recharge_list_entity.dart';
import 'package:mmoo_base_recharge/entity/recharge_type_entity.dart';
import 'package:mmoo_base_recharge/recharge_config.dart';
import 'package:sp_util/sp_util.dart';

class ExamplePaymentQueueDelegate implements SKPaymentQueueDelegateWrapper {
  @override
  bool shouldContinueTransaction(
      SKPaymentTransactionWrapper transaction, SKStorefrontWrapper storefront) {
    return true;
  }

  @override
  bool shouldShowPriceConsent() {
    return false;
  }
}

///定义一个抽象类-充值
abstract class OnRechargePluginDelegate {
  ///吐司弹窗
  ///type
  ///     类型 1：取消支付
  ///     类型 2：支付失败
  ///     类型 3：重试弹窗-点击继续-1个按钮
  ///     类型 4：重试弹窗-点击继续-2个按钮
  void showToast(int type, [bool success = false]);

  ///吐司弹窗-接口请求错误
  void showErrorToast(res);

  ///提示弹窗-返回弹窗实例、需要控制层级
  ///type
  ///     类型 1：支付失败-苹果调取和返回
  ///     类型 2：重试弹窗-接口重试
  ///     类型 3：重试弹窗-显示反馈
  ///     类型 4：重试弹窗-只显示重试
  Future showDialogBase(int type);

  ///转圈提示框
  Future showLoadingDialog();

  ///关闭提示框
  void popDialog();

  ///跳转网页支付
  Future navigatorPushWebView(String url, [int type = 0]);

  ///支付成功加币
  void rechargeSuccess(String? money, String? addMoney, String? price);

  ///埋点统计
  void logEvent(String name);
}

///单例的公共类-系统配置，充值，礼物
///   main.dart中设置
///   if (Platform.isAndroid) {
///     InAppPurchaseAndroidPlatformAddition.enablePendingPurchases();
///   }
///   初始化 initInAppPurchase
class RechargeUtils {
  factory RechargeUtils() => _getInstance();

  static RechargeUtils get instance => _getInstance();
  static RechargeUtils? _instance;

  RechargeUtils._internal();

  static RechargeUtils _getInstance() {
    _instance ??= RechargeUtils._internal();
    return _instance!;
  }

  void dispose() {
    timer?.cancel();
  }

  bool showLoading = false;
  bool showInPay = false;

  ///支付
  Map<RechargeTypeList, List<RechargeList>> rechargeModels = {};

  String? orderId;
  String? payPrice;

  String? payInType; //内购列表还是弹窗

  String? rechargeError;
  String? meUser;

  late InAppPurchase connection = InAppPurchase.instance;
  late StreamSubscription<List<PurchaseDetails>> subscription;

  Timer? timer;
  bool restored = false;
  int maxIndex = 5;

  late OnRechargePluginDelegate delegate;

  void initInAppPurchase(OnRechargePluginDelegate delegate) {
    try {
      this.delegate = delegate;
      Stream purchaseUpdated = connection.purchaseStream;
      subscription = purchaseUpdated.listen((purchaseDetailsList) {
        listenToPurchaseUpdated(purchaseDetailsList);
      }, onDone: () {
        subscription.cancel();
        AppConfig.printLog('subscription: cancel');
      }, onError: (error) {
        AppConfig.printLog('purchaseUpdated: $error');
      }) as StreamSubscription<List<PurchaseDetails>>;
      queryPastPurchases(first: true);
      if (Platform.isIOS) {
        var iosPlatformAddition = connection
            .getPlatformAddition<InAppPurchaseStoreKitPlatformAddition>();
        iosPlatformAddition.setDelegate(ExamplePaymentQueueDelegate());
      }
    } catch (e) {
      AppConfig.printLog(e);
    }
  }

  Future<void> listenToPurchaseUpdated(
      List<PurchaseDetails?> purchaseDetailsList) async {
    AppConfig.printLog(
        'listenToPurchaseUpdated: ${purchaseDetailsList.length}');
    if (purchaseDetailsList.isNotEmpty) {
      purchaseDetailsList = purchaseDetailsList.reversed.toList();
      Map payPurchase = {};
      for (PurchaseDetails? purchaseDetail in purchaseDetailsList) {
        if (purchaseDetail!.status == PurchaseStatus.purchased ||
            purchaseDetail.status == PurchaseStatus.restored) {
          String md5Str = md5
              .convert(const Utf8Encoder().convert(
                  purchaseDetail.verificationData.serverVerificationData))
              .toString();
          AppConfig.printLog('purchaseUpdated md5Str: $md5Str');
          if (payPurchase[purchaseDetail.productID] != md5Str) {
            payPurchase[purchaseDetail.productID] = md5Str;
            await purchaseUpdated(purchaseDetail);
            // } else if (payPurchase[purchaseDetail.productID] == md5Str) {
            //   completePurchaseInPay(purchaseDetail);
          }
        } else {
          await purchaseUpdated(purchaseDetail);
        }
      }
    } else if (restored) {
      popDialog();
      delegate.logEvent('${payInType ?? ''}回调为空清空本地缓存');
      SpUtil.putString(RechargeConfig.inappToken, '');
      SpUtil.putString(RechargeConfig.inappSign, '');
      SpUtil.putString(RechargeConfig.inappOrderId, '');
      SpUtil.putString(RechargeConfig.inappPrice, '');
      SpUtil.putString(RechargeConfig.inappCode, '');
      restored = false;
    }
  }

  Future purchaseUpdated(PurchaseDetails purchaseDetail) async {
    if (purchaseDetail.status == PurchaseStatus.pending) {
      if (purchaseDetail.productID.startsWith(RechargeConfig.subStart)) {
        delegate.logEvent('发起订阅');
      } else {
        delegate.logEvent('${payInType ?? ''}发起支付');
      }
      AppConfig.printLog('PurchaseStatus.pending');
    } else if (purchaseDetail.status == PurchaseStatus.error) {
      popDialog();
      AppConfig.printLog(
          'PurchaseStatus.error:${purchaseDetail.error?.message}');
      if (purchaseDetail.error?.details is Map &&
              purchaseDetail.error?.details['NSUnderlyingError'] is Map &&
              purchaseDetail.error?.details['NSUnderlyingError']['code'] ==
                  907 ||
          purchaseDetail.error?.message == 'BillingResponse.userCanceled') {
        if (purchaseDetail.productID.startsWith(RechargeConfig.subStart)) {
          delegate.logEvent('取消订阅');
        } else {
          delegate.logEvent('${payInType ?? ''}取消支付');
        }
        delegate.showToast(1);
        var res = await RechargeDao.rechargeCancel(orderId);
        if (res.result) {
          completePurchaseInPay(purchaseDetail);
        }
        orderId = '';
      } else if (purchaseDetail.error?.message ==
          'BillingResponse.itemAlreadyOwned') {
        queryPastPurchases();
      } else {
        if (purchaseDetail.productID.startsWith(RechargeConfig.subStart)) {
          delegate.logEvent('支付失败');
        } else {
          delegate.logEvent('${payInType ?? ''}支付失败');
        }
        showInPay = true;
        delegate.showDialogBase(1).then((value) async {
          showInPay = false;
        });
        var res = await RechargeDao.rechargeFailLog(
            '', '', orderId, purchaseDetail.error?.message);
        if (res.result) {
          completePurchaseInPay(purchaseDetail);
        }
      }
    } else if (purchaseDetail.status == PurchaseStatus.purchased ||
        purchaseDetail.status == PurchaseStatus.restored) {
      AppConfig.printLog('PurchaseStatus.purchased: ${purchaseDetail.status}');
      if (purchaseDetail.status == PurchaseStatus.restored) {
        restored = false;
      }
      if (orderId?.isEmpty != false) {
        orderId = SpUtil.getString(RechargeConfig.inappOrderId);
        payPrice = SpUtil.getString(RechargeConfig.inappPrice);
      }
      if (purchaseDetail.productID.startsWith(RechargeConfig.subStart)) {
        delegate.logEvent('订阅成功');
        SpUtil.putString(RechargeConfig.inappCode, purchaseDetail.productID);
        if (Platform.isIOS) {
          SpUtil.putString(RechargeConfig.inappToken,
              purchaseDetail.verificationData.serverVerificationData);
          await checkInPayPurchaseVip(
              purchaseDetail.verificationData.serverVerificationData,
              orderId,
              purchaseDetail.productID,
              price: payPrice,
              first: purchaseDetail.status != PurchaseStatus.restored,
              purchaseDetail: purchaseDetail,
              index: purchaseDetail.status == PurchaseStatus.restored
                  ? maxIndex
                  : 1);
        } else {
          SpUtil.putString(
              RechargeConfig.inappToken,
              (purchaseDetail as GooglePlayPurchaseDetails)
                  .billingClientPurchase
                  .originalJson);
          SpUtil.putString(RechargeConfig.inappSign,
              purchaseDetail.billingClientPurchase.signature);
          await checkInPayPurchaseVip(
              purchaseDetail.billingClientPurchase.originalJson,
              orderId,
              purchaseDetail.productID,
              sign: purchaseDetail.billingClientPurchase.signature,
              price: payPrice,
              first: purchaseDetail.status != PurchaseStatus.restored,
              purchaseDetail: purchaseDetail,
              index: purchaseDetail.status == PurchaseStatus.restored
                  ? maxIndex
                  : 1);
        }
      } else {
        delegate.logEvent('${payInType ?? ''}支付成功');
        SpUtil.putString(RechargeConfig.inappCode, purchaseDetail.productID);
        if (Platform.isIOS) {
          SpUtil.putString(RechargeConfig.inappToken,
              purchaseDetail.verificationData.serverVerificationData);
          await checkInPayPurchase(
              purchaseDetail.verificationData.serverVerificationData,
              orderId,
              purchaseDetail.productID,
              price: payPrice,
              first: purchaseDetail.status != PurchaseStatus.restored,
              purchaseDetail: purchaseDetail,
              index: purchaseDetail.status == PurchaseStatus.restored
                  ? maxIndex
                  : 1);
        } else {
          SpUtil.putString(
              RechargeConfig.inappToken,
              (purchaseDetail as GooglePlayPurchaseDetails)
                  .billingClientPurchase
                  .originalJson);
          SpUtil.putString(RechargeConfig.inappSign,
              purchaseDetail.billingClientPurchase.signature);
          await checkInPayPurchase(
              purchaseDetail.billingClientPurchase.originalJson,
              orderId,
              purchaseDetail.productID,
              sign: purchaseDetail.billingClientPurchase.signature,
              price: payPrice,
              first: purchaseDetail.status != PurchaseStatus.restored,
              purchaseDetail: purchaseDetail,
              index: purchaseDetail.status == PurchaseStatus.restored
                  ? maxIndex
                  : 1);
        }
      }
    } else {
      popDialog();
    }
  }

  ///审核模式-mode:true审核模式，false正式模式
  Future<void> getRechargeList(changed,
      {String type = '0',
      bool mode = true,
      bool pop = true,
      bool cache = true}) async {
    if (mode) {
      if (cache && rechargeModels.isNotEmpty) {
        changed?.call(CommonUtils.pageStatusSuccess);
        return;
      }
      clearRechargeList();
      showLoading = false;
      changed?.call(CommonUtils.pageStatusLoading);
      RechargeTypeList codeType = RechargeTypeList()
        ..type = Platform.isAndroid ? '2' : '1';
      bool ok = await rechargeInList(type, codeType, pop: pop);
      changed?.call(rechargeModels.isEmpty
          ? CommonUtils.pageStatusEmpty
          : ok
              ? CommonUtils.pageStatusSuccess
              : CommonUtils.pageStatusError);
    } else {
      showLoading = false;
      changed?.call(CommonUtils.pageStatusLoading);
      RechargeDao.rechargeSupport().then((res) async {
        if (res.result) {
          List<RechargeTypeList> list = res.data.rechargeModels;
          if (list.isNotEmpty) {
            clearRechargeList();
            bool ok = false;
            for (RechargeTypeList item in list) {
              ok = await rechargeInList(type, item, pop: pop);
              if (!ok) {
                changed?.call(CommonUtils.pageStatusError);
                return;
              }
            }
          } else {
            clearRechargeList();
          }
        }
        changed?.call(rechargeModels.isEmpty
            ? CommonUtils.pageStatusEmpty
            : CommonUtils.pageStatusSuccess);
      });
    }
  }

  Future rechargeInList(String type, RechargeTypeList codeType,
      {bool pop = false}) {
    return RechargeDao.rechargeList(type, codeType.type).then((res) {
      if (res.result) {
        List<RechargeList> list = res.data.list;
        list.sort((a, b) =>
            compareString(a.sortTop).compareTo(compareString(b.sortTop)));
        rechargeModels[codeType] = list;
        return true;
      }
      return false;
    });
  }

  void clearRechargeList() {
    rechargeModels.clear();
  }

  Future queryPastPurchases({bool first = false, bool refresh = false}) async {
    if (await connection.isAvailable()) {
      if (Platform.isAndroid || refresh) {
        if (restored) {
          delegate.logEvent('${payInType ?? ''}票据为空清空本地缓存');
          SpUtil.putString(RechargeConfig.inappToken, '');
          SpUtil.putString(RechargeConfig.inappSign, '');
          SpUtil.putString(RechargeConfig.inappOrderId, '');
          SpUtil.putString(RechargeConfig.inappPrice, '');
          SpUtil.putString(RechargeConfig.inappCode, '');
          restored = false;
        } else {
          restored = true;
          await connection.restorePurchases();
        }
      } else if (!first) {
        InAppPurchaseStoreKitPlatformAddition iosAddition = connection
            .getPlatformAddition<InAppPurchaseStoreKitPlatformAddition>();
        PurchaseVerificationData? verificationData =
            await iosAddition.refreshPurchaseVerificationData();
        if (verificationData != null) {
          SpUtil.putString(RechargeConfig.inappToken,
              verificationData.serverVerificationData);
          await checkInPayPurchase(
              verificationData.serverVerificationData, '', '',
              first: true, refresh: true);
          // await checkInPayPurchaseVip(
          //     verificationData.serverVerificationData, '', '',
          //     first: true,
          //     refresh: true);
        } else {
          delegate.logEvent('${payInType ?? ''}票据为空清空本地缓存');
          SpUtil.putString(RechargeConfig.inappToken, '');
          SpUtil.putString(RechargeConfig.inappSign, '');
          SpUtil.putString(RechargeConfig.inappOrderId, '');
          SpUtil.putString(RechargeConfig.inappPrice, '');
          SpUtil.putString(RechargeConfig.inappCode, '');
        }
      }
    }
  }

  Future completePurchaseInPay(PurchaseDetails? purchaseDetail) async {
    try {
      if (Platform.isAndroid) {
        InAppPurchaseAndroidPlatformAddition androidAddition = InAppPurchase
            .instance
            .getPlatformAddition<InAppPurchaseAndroidPlatformAddition>();
        BillingResultWrapper billingResult =
            await androidAddition.consumePurchase(purchaseDetail!);
        AppConfig.printLog('completePurchase: ${billingResult.responseCode}');
      } else if (purchaseDetail?.pendingCompletePurchase == true) {
        await connection.completePurchase(purchaseDetail!);
        AppConfig.printLog('completePurchase: checkInPayPurchase');
      } else {
        paymentQueueWrapper(payId: purchaseDetail?.productID, delayed: false);
      }
    } catch (e) {
      AppConfig.printLog(e);
    }
  }

  Future paymentQueueWrapper(
      {String? payId, bool delayed = true, bool finish = true}) async {
    if (Platform.isIOS) {
      var transactions = await SKPaymentQueueWrapper().transactions();
      if (payId != null) {
        transactions = transactions.skipWhile((value) {
          return payId.isEmpty == false &&
              payId != value.payment.productIdentifier;
        }).toList();
      }
      AppConfig.printLog('paymentQueueWrapper: ${transactions.length}');
      if (transactions.isEmpty == false) {
        if (finish) {
          AppConfig.printLog(
              'paymentQueueWrapper finishTransaction: ${transactions.length}');
          for (SKPaymentTransactionWrapper wrapper in transactions) {
            await SKPaymentQueueWrapper().finishTransaction(wrapper);
          }
        }
        if (delayed) {
          await Future.delayed(const Duration(seconds: 5), () async {
            await paymentQueueWrapper(finish: finish);
          });
        }
      }
    }
  }

  ///单独使用支付等
  Future connectionPP(String? payId, String? price, {back}) {
    if (!showLoading) {
      showLoading = true;
      delegate.showLoadingDialog();
      return RechargeDao.rechargeOrder(payId).then((res) {
        popDialog();
        if (res.result) {
          orderId = res.data.id;
          payPrice = price;
          delegate.logEvent('发起paypal支付');
          back?.call(true);
          delegate.navigatorPushWebView(res.data.url).then((value) {
            back?.call(false);
          });
        } else {
          delegate.showErrorToast(res);
        }
      });
    } else {
      return Future(() {});
    }
  }

  Future connectionTBF(String? payId, String? price, req, {back}) {
    if (!showLoading) {
      showLoading = true;
      delegate.showLoadingDialog();
      return RechargeDao.rechargeOrder(payId, req: req).then((res) {
        if (res.result) {
          if (res.data.result.orderSucceed == 0) {
            popDialog();
            delegate.showErrorToast(res.data.result.orderResult);
            return -2;
          } else {
            orderId = res.data.id;
            payPrice = price;
            if (res.data.result.redirectURL?.isEmpty == false) {
              popDialog();
              delegate.logEvent('发起tbf3d支付');
              back?.call(true);
              delegate
                  .navigatorPushWebView(res.data.result.redirectURL, 1)
                  .then((value) {
                back?.call(false);
              });
              return -2;
            }
            return res.data.result.orderSucceed;
          }
        } else {
          popDialog();
          delegate.showErrorToast(res);
          return -2;
        }
      });
    } else {
      return Future(() {});
    }
  }

  Future connectionInPay(String? id, String? payId, String? price,
      {String? inType}) async {
    if (await checkInPay(first: false) == false && !showLoading) {
      if (inType != null) {
        payInType = inType;
      } else {
        payInType = null;
      }
      if (!showLoading) {
        showLoading = true;
        delegate.showLoadingDialog();
      }
      final bool isAvailable = await connection.isAvailable();
      if (isAvailable) {
        Set<String> _kProductIds = {payId!};
        ProductDetailsResponse productDetailResponse =
            await connection.queryProductDetails(_kProductIds);
        if (productDetailResponse.error == null) {
          List<ProductDetails> products = productDetailResponse.productDetails;
          if (products.isNotEmpty) {
            return RechargeDao.rechargeOrder(id).then((res) async {
              if (res.result) {
                bool buy = false;
                try {
                  PurchaseParam purchaseParam =
                      PurchaseParam(productDetails: products[0]);
                  buy = await connection.buyConsumable(
                      purchaseParam: purchaseParam,
                      autoConsume: !Platform.isAndroid);
                  AppConfig.printLog('buyConsumable: $buy');
                  if (buy) {
                    orderId = res.data.id;
                    payPrice = price;
                    SpUtil.putString(RechargeConfig.inappOrderId, orderId!);
                    SpUtil.putString(RechargeConfig.inappPrice, payPrice!);
                    return true;
                  }
                } catch (e) {
                  delegate.logEvent('${payInType ?? ''}发起支付失败');
                  AppConfig.printLog(e);
                  popDialog();
                  showInPay = true;
                  delegate.showDialogBase(1).then((value) async {
                    showInPay = false;
                    if (value == true) {
                      queryPastPurchases(refresh: true);
                    }
                  });
                  RechargeDao.rechargeFailLog('', '', orderId, e.toString());
                  return false;
                }
              }
              delegate.logEvent('${payInType ?? ''}统一下单失败');
              popDialog();
              delegate.showErrorToast(res);
              return false;
            });
          }
        }
      }
      delegate.logEvent('${payInType ?? ''}支付不存在');
      popDialog();
      delegate.showToast(2);
      return false;
    } else {
      return false;
    }
  }

  void connectionInPayVip(String id, String? payId, String price) async {
    if (await checkInPay(first: false) == false && !showLoading) {
      if (!showLoading) {
        showLoading = true;
        delegate.showLoadingDialog();
      }
      final bool isAvailable = await connection.isAvailable();
      if (isAvailable) {
        Set<String> _kProductIds = {payId!};
        ProductDetailsResponse productDetailResponse =
            await connection.queryProductDetails(_kProductIds);
        if (productDetailResponse.error == null) {
          List<ProductDetails> products = productDetailResponse.productDetails;
          if (products.isNotEmpty) {
            bool buy = false;
            try {
              PurchaseParam purchaseParam =
                  PurchaseParam(productDetails: products[0]);
              buy = await connection.buyNonConsumable(
                  purchaseParam: purchaseParam);
              AppConfig.printLog('buyNonConsumable: $buy');
              if (buy) {
                orderId = '';
                payPrice = price;
                SpUtil.putString(RechargeConfig.inappOrderId, orderId!);
                SpUtil.putString(RechargeConfig.inappPrice, payPrice!);
                delegate.logEvent('发起订阅');
                return;
              }
            } catch (e) {
              delegate.logEvent('发起订阅失败');
              AppConfig.printLog(e);
              popDialog();
              showInPay = true;
              delegate.showDialogBase(1).then((value) async {
                showInPay = false;
                if (value == true) {
                  queryPastPurchases(refresh: true);
                }
              });
              RechargeDao.rechargeFailLog('', '', orderId, e.toString());
              return;
            }
          }
        }
      }
      delegate.logEvent('订阅不存在');
      popDialog();
      delegate.showToast(2);
    }
  }

  Future checkInPayPurchase(String? token, String? orderId, String? productID,
      {String? sign,
      String? price,
      bool first = false,
      bool refresh = false,
      PurchaseDetails? purchaseDetail,
      int index = 1,
      int indexNum = 1,
      int indexNumNext = 1}) async {
    String? baseToken = SpUtil.getString(AppConfig.appToken);
    if (baseToken?.isEmpty == false) {
      await RechargeDao.rechargeInCheck(token, sign, productID: productID)
          .then((res) async {
        timer?.cancel();
        if (res.result) {
          this.orderId = '';
          bool show = popDialog();
          if ((res.data.addMoney?.compareTo('0') ?? 0) > 0) {
            if (show) {
              delegate.showToast(3, true);
            }
            clearRechargeList();
            delegate.logEvent('${payInType ?? ''}加币成功');
          } else {
            delegate.logEvent('${payInType ?? ''}重复加币');
          }
          delegate.rechargeSuccess(res.data.money, res.data.addMoney, price);
          SpUtil.putString(RechargeConfig.inappToken, '');
          SpUtil.putString(RechargeConfig.inappSign, '');
          SpUtil.putString(RechargeConfig.inappOrderId, '');
          SpUtil.putString(RechargeConfig.inappPrice, '');
          SpUtil.putString(RechargeConfig.inappCode, '');
          if (purchaseDetail != null) {
            completePurchaseInPay(purchaseDetail);
          } else if (Platform.isAndroid) {
            queryPastPurchases();
          } else {
            paymentQueueWrapper(payId: productID, delayed: false);
          }
        } else if (res.data.code == '1068' && Platform.isIOS) {
          if (refresh) {
            delegate.logEvent('服务器1068清空本地缓存');
            SpUtil.putString(RechargeConfig.inappToken, '');
            SpUtil.putString(RechargeConfig.inappSign, '');
            SpUtil.putString(RechargeConfig.inappOrderId, '');
            SpUtil.putString(RechargeConfig.inappPrice, '');
            SpUtil.putString(RechargeConfig.inappCode, '');
          } else if (index == 1) {
            queryPastPurchases();
          } else {
            popDialog();
            showInPay = true;
            delegate.showDialogBase(2).then((value) async {
              showInPay = false;
              if (value == true) {
                queryPastPurchases();
              }
            });
          }
        } else {
          delegate.logEvent('支付重试$index');
          try {
            rechargeError = res.data?.msg;
          } catch (e) {
            AppConfig.printLog(e);
          }
          var connectivityResult = await (Connectivity().checkConnectivity());
          AppConfig.printLog(
              '支付失败重试: $index, 时间: ${index > 2 ? indexNum + indexNumNext : indexNumNext}');
          if (connectivityResult == ConnectivityResult.none ||
              index >= maxIndex) {
            delegate.showErrorToast(res);
            popDialog();
            showInPayDialog(token, orderId, productID,
                sign: sign,
                price: price,
                showLeft: !first,
                restored: purchaseDetail?.status == PurchaseStatus.restored);
          } else {
            timer = Timer(
                Duration(
                    seconds: index > 2
                        ? indexNum + indexNumNext
                        : indexNumNext), () async {
              checkInPayPurchase(token, orderId, productID,
                  sign: sign,
                  price: price,
                  first: first,
                  purchaseDetail: purchaseDetail,
                  index: index + 1,
                  indexNum: indexNumNext,
                  indexNumNext:
                      index > 2 ? indexNum + indexNumNext : indexNumNext);
            });
          }
        }
      });
    } else {
      delegate.logEvent('支付成功token空');
    }
  }

  Future checkInPayPurchaseVip(
      String? token, String? orderId, String? productID,
      {String? sign,
      String? price,
      bool first = false,
      bool refresh = false,
      PurchaseDetails? purchaseDetail,
      int index = 1,
      int indexNum = 1,
      int indexNumNext = 1}) async {
    String? baseToken = SpUtil.getString(AppConfig.appToken);
    if (baseToken?.isEmpty == false) {
      await RechargeDao.rechargeInCheckVip(token, sign, orderId,
              productID: productID)
          .then((res) async {
        timer?.cancel();
        if (res.result) {
          this.orderId = '';
          bool show = popDialog();
          if ((res.data.addMoney?.compareTo('0') ?? 0) > 0) {
            if (show) {
              delegate.showToast(4, true);
            }
            clearRechargeList();
            delegate.logEvent('订阅加币成功');
          } else {
            delegate.logEvent('订阅重复加币');
          }
          delegate.rechargeSuccess(res.data.money, res.data.addMoney, price);
          SpUtil.putString(RechargeConfig.inappToken, '');
          SpUtil.putString(RechargeConfig.inappSign, '');
          SpUtil.putString(RechargeConfig.inappOrderId, '');
          SpUtil.putString(RechargeConfig.inappPrice, '');
          SpUtil.putString(RechargeConfig.inappCode, '');
          if (purchaseDetail != null) {
            completePurchaseInPay(purchaseDetail);
          } else if (Platform.isAndroid) {
            queryPastPurchases();
          } else {
            paymentQueueWrapper(payId: productID, delayed: false);
          }
        } else if (res.data.code == '1068' && Platform.isIOS) {
          if (refresh) {
            delegate.logEvent('订阅服务器1068清空本地缓存');
            SpUtil.putString(RechargeConfig.inappToken, '');
            SpUtil.putString(RechargeConfig.inappSign, '');
            SpUtil.putString(RechargeConfig.inappOrderId, '');
            SpUtil.putString(RechargeConfig.inappPrice, '');
            SpUtil.putString(RechargeConfig.inappCode, '');
          } else if (index == 1) {
            queryPastPurchases();
          } else {
            popDialog();
            showInPay = true;
            delegate.showDialogBase(2).then((value) async {
              showInPay = false;
              if (value == true) {
                queryPastPurchases();
              }
            });
          }
        } else {
          delegate.logEvent('订阅重试$index');
          try {
            rechargeError = res.data?.msg;
          } catch (e) {
            AppConfig.printLog(e);
          }
          var connectivityResult = await (Connectivity().checkConnectivity());
          AppConfig.printLog(
              '订阅失败重试: $index, 时间: ${index > 2 ? indexNum + indexNumNext : indexNumNext}');
          if (connectivityResult == ConnectivityResult.none ||
              index >= maxIndex) {
            delegate.showErrorToast(res);
            popDialog();
            showInPayDialog(token, orderId, productID,
                sign: sign,
                price: price,
                showLeft: !first,
                restored: purchaseDetail?.status == PurchaseStatus.restored);
          } else {
            timer = Timer(
                Duration(
                    seconds: index > 2
                        ? indexNum + indexNumNext
                        : indexNumNext), () async {
              checkInPayPurchaseVip(token, orderId, productID,
                  sign: sign,
                  price: price,
                  first: first,
                  purchaseDetail: purchaseDetail,
                  index: index + 1,
                  indexNum: indexNumNext,
                  indexNumNext:
                      index > 2 ? indexNum + indexNumNext : indexNumNext);
            });
          }
        }
      });
    } else {
      delegate.logEvent('订阅成功token空');
    }
  }

  //return 0接口失败 -1取消 1支付中 2加币成功 3重复加币
  //type 0paypal 1card
  Future checkPpyPurchase(orderId,
      {bool show = false,
      int type = 0,
      int index = 1,
      int indexNum = 1,
      int indexNumNext = 1}) {
    if (show && !showLoading) {
      showLoading = true;
      delegate.showLoadingDialog();
    }
    return Future.delayed(
        Duration(seconds: index > 2 ? indexNum + indexNumNext : indexNumNext),
        () async {
      return RechargeDao.rechargePpCheck(orderId).then((res) {
        if (res.result) {
          if (res.data.payed == '1') {
            popDialog();
            this.orderId = '';
            if ((res.data.addMoney?.compareTo('0') ?? 0) > 0) {
              delegate.showToast(3, true);
              clearRechargeList();
              delegate.rechargeSuccess(
                  res.data.money, res.data.addMoney, payPrice);
              delegate.logEvent((type == 0 ? 'paypal' : '信用卡支付') + '加币成功');
              return 2;
            }
            delegate.rechargeSuccess(
                res.data.money, res.data.addMoney, payPrice);
            delegate.logEvent((type == 0 ? 'paypal' : '信用卡支付') + '重复加币');
            return 3;
          } else if (show && res.data.payed == '2') {
            AppConfig.printLog(
                '支付失败重试: $index, 时间: ${index > 2 ? indexNum + indexNumNext : indexNumNext}');
            delegate.logEvent((type == 0 ? 'paypal' : '信用卡支付') + '支付重试$index');
            return checkPpyPurchase(orderId,
                show: show,
                type: type,
                index: index + 1,
                indexNum: indexNumNext,
                indexNumNext:
                    index > 2 ? indexNum + indexNumNext : indexNumNext);
          } else if (!show && res.data.payed == '0') {
            popDialog();
            RechargeDao.rechargeCancel(orderId);
            this.orderId = '';
            return -1;
          } else {
            popDialog();
            this.orderId = '';
            return 0;
          }
        } else {
          try {
            rechargeError = res.data?.msg;
          } catch (e) {
            AppConfig.printLog(e);
          }
          delegate.showErrorToast(res);
        }
        popDialog();
        return 0;
      });
    });
  }

  Future checkInPay({bool first = true}) async {
    orderId = SpUtil.getString(RechargeConfig.inappOrderId);
    String? token = SpUtil.getString(RechargeConfig.inappToken);
    if (orderId?.isEmpty == false && token?.isEmpty == false) {
      payPrice = SpUtil.getString(RechargeConfig.inappPrice);
      String? productID = SpUtil.getString(RechargeConfig.inappCode);
      if (first) {
        startInPayPurchase(token, orderId, productID,
            sign: SpUtil.getString(RechargeConfig.inappSign),
            price: payPrice,
            first: first);
      } else {
        showInPayDialog(token, orderId, productID,
            sign: SpUtil.getString(RechargeConfig.inappSign),
            price: payPrice,
            showLeft: first);
      }
      return true;
    }
    return false;
  }

  void showInPayDialog(String? token, String? orderId, String? productID,
      {String? sign,
      String? price,
      bool showLeft = true,
      bool restored = false}) {
    if (showInPay || restored) {
      return;
    }
    showInPay = true;
    delegate.showDialogBase(showLeft ? 3 : 4).then((value) async {
      showInPay = false;
      if (value == true) {
        startInPayPurchase(token, orderId, productID, sign: sign, price: price);
      } else if (value == null) {
        if (!showLoading) {
          showLoading = true;
          delegate.showLoadingDialog();
          RechargeDao.rechargeFailLog(token, sign, orderId, rechargeError)
              .then((res) async {
            popDialog();
            if (!res.result) {
              delegate.showErrorToast(res);
            }
          });
        }
      }
    });
  }

  void startInPayPurchase(String? token, String? orderId, String? productID,
      {String? sign,
      String? price,
      bool first = false,
      int index = 1,
      int indexNum = 1,
      int indexNumNext = 1}) async {
    if (!showLoading) {
      if (!first) {
        showLoading = true;
        delegate.showLoadingDialog();
        if (productID!.startsWith(RechargeConfig.subStart)) {
          checkInPayPurchaseVip(token, orderId, productID,
              sign: sign,
              price: price,
              first: first,
              index: index,
              indexNum: indexNum,
              indexNumNext: indexNumNext);
        } else {
          checkInPayPurchase(token, orderId, productID,
              sign: sign,
              price: price,
              first: first,
              index: index,
              indexNum: indexNum,
              indexNumNext: indexNumNext);
        }
      } else {
        if (productID!.startsWith(RechargeConfig.subStart)) {
          checkInPayPurchaseVip(token, orderId, productID,
              sign: sign,
              price: price,
              first: first,
              index: index,
              indexNum: indexNum,
              indexNumNext: indexNumNext);
        } else {
          checkInPayPurchase(token, orderId, productID,
              sign: sign,
              price: price,
              first: first,
              index: index,
              indexNum: indexNum,
              indexNumNext: indexNumNext);
        }
      }
    }
  }

  String compareString(String? str) {
    return str?.length == 1 ? '0$str' : (str ?? '');
  }

  bool popDialog() {
    if (showLoading) {
      showLoading = false;
      delegate.popDialog();
      return true;
    }
    return false;
  }
}
